;==============================================
;  mand87.asm
;
;    Requires MASM 6.0, 386/387 or i486, MS-DOS
;
;    Assembly language implementation of a fast
;    Mandelbrot generator.
;
;    Copyright 1993 Scott Robert Ladd.
;    All Rights Reserved.
;==============================================

;-----------------------------------
; memory model and processor options
;-----------------------------------

.MODEL SMALL
.386
.387

;--------------
; stack segment
;--------------

.STACK 128

;--------------
; data segments
;--------------

.CONST
    XPixels WORD   320
    YPixels WORD   200
    MaxIter WORD   150 

    XFactor REAL8  0.003125
    YFactor REAL8  0.005000
    XMin    REAL8 -0.64
    XMax    REAL8 -0.44
    YMin    REAL8  0.55
    YMax    REAL8  0.70
    Escape  REAL8  4.0

.DATA?
    DeltaX  REAL8  ?
    DeltaY  REAL8  ?
    RowVal  REAL8  ?
    Xv      REAL8  ?
    XTable  REAL8  200 DUP (?)

;------------------------------
; code segment and main program
;------------------------------

.CODE
.STARTUP

    fninit              ; reset the coprocessor

    mov     ax,0013h    ; set 320x200x256 mode
    int     10h

    mov     ax,0A000h   ; set-up graphics segment
    mov     es,ax

    fld     XMax        ; DeltaX = 
    fsub    XMin        ;    (XMax - XMin) 
    fmul    XFactor     ;      * XFactor
    fstp    DeltaX

    fld     YMax        ; DeltaY = 
    fsub    YMin        ;    (YMax - YMin)
    fmul    YFactor     ;      * YFactor
    fstp    DeltaY

    mov     di,2552     ; fill array of Y values

    fld     XMax        ; load first element

SetXTable:
    fst     XTable[di]  ; store element

    and     di,di       ; if di == 0
    jz      Setup       ;    we're done

    fsub    DeltaX      ; get next X value
    sub     di,8        ; move to next element
    jmp     SetXTable   ; loop

Setup:
    fstp    st          ; pop top of stack

    fld     YMax        ; set RowVal = YMin
    fstp    RowVal

    mov     dx,0        ; set row (dx) to 0
    mov     bx,0        ; set bx to point to first pixel

RowLoop:
    mov     si,0        ; set col (si) to zero

ColLoop:
    fld     RowVal      ; initial Y

    mov     di,si
    shl     di,3
    fld     XTable[di]  ; load X value from table
    fst     Xv          ; stays on stack as X

    mov     cx,0        ; cx contains color value

InnerLoop:
    fld     st(1)       ; put new squares on stack
    fmul    st,st
    fld     st(1)
    fmul    st,st

    fld     st(1)       ; if (Xsq + Ysq) > Escape
    fadd    st,st(1)    ;    plot pixel
    fcomp   Escape
    fnstsw  ax
    test    ax,8100h
    jz      ClearAndPlot

    fld     st(3)       ; Y = 2 * (Y * X) + RowVal
    fmul    st,st(3)
    fadd    st,st
    fadd    RowVal
    fstp    st(4)

    fld     st          ; X = Xsq - Ysq + Xv
    fsub    st,st(2)
    fadd    Xv
    fstp    st(3)

    fstp    st          ; remove old squares
    fstp    st

    inc     cx          ; increment count

    cmp     cx,MaxIter  ; if color < MaxIter
    jl      InnerLoop   ;    continue looping

    mov     BYTE PTR es:[bx],0 ; plot black pixel
    jmp     PlotPixel

ClearAndPlot:
    fstp    st          ; remove old squares
    fstp    st

    and     cx,15       ; set color
    inc     cx
    mov     es:[bx],cl

PlotPixel:
    fstp    st          ; remove X and Y
    fstp    st
    
    inc     bx          ; goto next pixel

    inc     si          ; increment column 
    cmp     si,XPixels  ;         and loop
    jl      ColLoop

    fld     RowVal      ; calculate next RowVal
    fsub    DeltaY
    fstp    RowVal

    inc     dx          ; increment row
    cmp     dx,YPixels  ;         and loop
    jl      RowLoop

    mov     ax,0003h    ; set text display mode
    int     10h

    mov     ax,4C00h    ; exit program
    int     21h

END

